import { v4 as uuidv4 } from 'uuid';
var BRACES = {
  '[': ']',
  '{': '}',
  '(': ')'
};
var MATCH_MARK = 'brace_match';
export function BracesPlugin() {
  return {
    onKeyDown: function onKeyDown(event, editor, next) {
      var keyEvent = event;
      var value = editor.value;

      switch (keyEvent.key) {
        case '(':
        case '{':
        case '[':
          {
            var _value$selection = value.selection,
                _value$selection$star = _value$selection.start,
                startOffset = _value$selection$star.offset,
                startKey = _value$selection$star.key,
                _value$selection$end = _value$selection.end,
                endOffset = _value$selection$end.offset,
                endKey = _value$selection$end.key,
                focusOffset = _value$selection.focus.offset;
            var text = value.focusText.text; // If text is selected, wrap selected text in parens

            if (value.selection.isExpanded) {
              keyEvent.preventDefault();
              editor.insertTextByKey(startKey, startOffset, keyEvent.key).insertTextByKey(endKey, endOffset + 1, BRACES[keyEvent.key]).moveEndBackward(1);
              return true;
            } else if ( // Insert matching brace when there is no input after caret
            focusOffset === text.length || text[focusOffset] === ' ' || Object.values(BRACES).includes(text[focusOffset])) {
              keyEvent.preventDefault();
              var complement = BRACES[keyEvent.key];
              var matchAnnotation = {
                key: "".concat(MATCH_MARK, "-").concat(uuidv4()),
                type: "".concat(MATCH_MARK, "-").concat(complement),
                anchor: {
                  key: startKey,
                  offset: startOffset,
                  object: 'point'
                },
                focus: {
                  key: endKey,
                  offset: endOffset + 1,
                  object: 'point'
                },
                object: 'annotation'
              };
              editor.insertText(keyEvent.key).insertText(complement).addAnnotation(matchAnnotation).moveBackward(1);
              return true;
            }

            break;
          }

        case ')':
        case '}':
        case ']':
          {
            var _text = value.anchorText.text;
            var offset = value.selection.anchor.offset;
            var nextChar = _text[offset]; // Handle closing brace when it's already the next character

            var _complement = keyEvent.key;
            var annotationType = "".concat(MATCH_MARK, "-").concat(_complement);
            var annotation = value.annotations.find(function (a) {
              return (a === null || a === void 0 ? void 0 : a.type) === annotationType && a.anchor.key === value.anchorText.key;
            });

            if (annotation && nextChar === _complement && !value.selection.isExpanded) {
              keyEvent.preventDefault();
              editor.moveFocusForward(1).removeAnnotation(annotation).moveAnchorForward(1);
              return true;
            }

            break;
          }

        case 'Backspace':
          {
            var _text2 = value.anchorText.text;
            var _offset = value.selection.anchor.offset;
            var previousChar = _text2[_offset - 1];
            var _nextChar = _text2[_offset];

            if (BRACES[previousChar] && BRACES[previousChar] === _nextChar) {
              keyEvent.preventDefault(); // Remove closing brace if directly following

              editor.deleteBackward(1).deleteForward(1).focus();
              return true;
            }
          }

        default:
          {
            break;
          }
      }

      return next();
    }
  };
}