function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import Prism from 'prismjs';
import { Block } from 'slate';
import Options from './options';
import TOKEN_MARK from './TOKEN_MARK';
/**
 * A Slate plugin to highlight code syntax.
 */

export function SlatePrism() {
  var optsParam = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var prismLanguages = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Prism.languages;
  var opts = new Options(optsParam);
  return {
    decorateNode: function decorateNode(node, editor, next) {
      if (!opts.onlyIn(node)) {
        return next();
      }

      var block = Block.create(node);
      var grammarName = opts.getSyntax(block);
      var grammar = prismLanguages[grammarName];

      if (!grammar) {
        // Grammar not loaded
        return [];
      } // Tokenize the whole block text


      var texts = block.getTexts();
      var blockText = texts.map(function (text) {
        return text && text.getText();
      }).join('\n');
      var tokens = Prism.tokenize(blockText, grammar);
      var flattened = flattenTokens(tokens);
      var newData = editor.value.data.set('tokens', flattened);
      editor.setData(newData);
      return _decorateNode(opts, tokens, block);
    },
    renderDecoration: function renderDecoration(props, editor, next) {
      return opts.renderDecoration({
        children: props.children,
        decoration: props.decoration
      }, editor, next);
    }
  };
}
/**
 * Returns the decoration for a node
 */

function _decorateNode(opts, tokens, block) {
  var texts = block.getTexts(); // The list of decorations to return

  var decorations = [];
  var textStart = 0;
  var textEnd = 0;
  texts.forEach(function (text) {
    textEnd = textStart + text.getText().length;
    var offset = 0;

    function processToken(token, accu) {
      if (typeof token === 'string') {
        if (accu) {
          var decoration = createDecoration({
            text: text,
            textStart: textStart,
            textEnd: textEnd,
            start: offset,
            end: offset + token.length,
            className: "prism-token token ".concat(accu),
            block: block
          });

          if (decoration) {
            decorations.push(decoration);
          }
        }

        offset += token.length;
      } else {
        accu = "".concat(accu, " ").concat(token.type);

        if (token.alias) {
          accu += ' ' + token.alias;
        }

        if (typeof token.content === 'string') {
          var _decoration = createDecoration({
            text: text,
            textStart: textStart,
            textEnd: textEnd,
            start: offset,
            end: offset + token.content.length,
            className: "prism-token token ".concat(accu),
            block: block
          });

          if (_decoration) {
            decorations.push(_decoration);
          }

          offset += token.content.length;
        } else {
          // When using token.content instead of token.matchedStr, token can be deep
          for (var i = 0; i < token.content.length; i += 1) {
            // @ts-ignore
            processToken(token.content[i], accu);
          }
        }
      }
    }

    tokens.forEach(processToken);
    textStart = textEnd + 1; // account for added `\n`
  });
  return decorations;
}
/**
 * Return a decoration range for the given text.
 */


function createDecoration(_ref) {
  var text = _ref.text,
      textStart = _ref.textStart,
      textEnd = _ref.textEnd,
      start = _ref.start,
      end = _ref.end,
      className = _ref.className,
      block = _ref.block;

  if (start >= textEnd || end <= textStart) {
    // Ignore, the token is not in the text
    return null;
  } // Shrink to this text boundaries


  start = Math.max(start, textStart);
  end = Math.min(end, textEnd); // Now shift offsets to be relative to this text

  start -= textStart;
  end -= textStart;
  var myDec = block.createDecoration({
    object: 'decoration',
    anchor: {
      key: text.key,
      offset: start,
      object: 'point'
    },
    focus: {
      key: text.key,
      offset: end,
      object: 'point'
    },
    type: TOKEN_MARK,
    data: {
      className: className
    }
  });
  return myDec;
}

function flattenToken(token) {
  if (typeof token === 'string') {
    return [{
      content: token,
      types: [],
      aliases: []
    }];
  } else if (Array.isArray(token)) {
    return token.flatMap(function (t) {
      return flattenToken(t);
    });
  } else if (token instanceof Prism.Token) {
    return flattenToken(token.content).flatMap(function (t) {
      var aliases = [];

      if (typeof token.alias === 'string') {
        aliases = [token.alias];
      } else {
        var _token$alias;

        aliases = (_token$alias = token.alias) !== null && _token$alias !== void 0 ? _token$alias : [];
      }

      return {
        content: t.content,
        types: [token.type].concat(_toConsumableArray(t.types)),
        aliases: [].concat(_toConsumableArray(aliases), _toConsumableArray(t.aliases))
      };
    });
  }

  return [];
}

export function flattenTokens(token) {
  var tokens = flattenToken(token);

  if (!tokens.length) {
    return [];
  }

  var firstToken = tokens[0];
  firstToken.prev = null;
  firstToken.next = tokens.length >= 2 ? tokens[1] : null;
  firstToken.offsets = {
    start: 0,
    end: firstToken.content.length
  };

  for (var i = 1; i < tokens.length - 1; i++) {
    tokens[i].prev = tokens[i - 1];
    tokens[i].next = tokens[i + 1];
    tokens[i].offsets = {
      start: tokens[i - 1].offsets.end,
      end: tokens[i - 1].offsets.end + tokens[i].content.length
    };
  }

  var lastToken = tokens[tokens.length - 1];
  lastToken.prev = tokens.length >= 2 ? tokens[tokens.length - 2] : null;
  lastToken.next = null;
  lastToken.offsets = {
    start: tokens.length >= 2 ? tokens[tokens.length - 2].offsets.end : 0,
    end: tokens.length >= 2 ? tokens[tokens.length - 2].offsets.end + lastToken.content.length : lastToken.content.length
  };
  return tokens;
}