function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import debounce from 'lodash/debounce';
import sortBy from 'lodash/sortBy';
import { TOKEN_MARK } from '../constants';
import { Typeahead } from '../Typeahead';
import { makeFragment } from '../utils';
export var TYPEAHEAD_DEBOUNCE = 250;
export function SuggestionsPlugin(_ref) {
  var onTypeahead = _ref.onTypeahead,
      cleanText = _ref.cleanText,
      onWillApplySuggestion = _ref.onWillApplySuggestion,
      portalOrigin = _ref.portalOrigin;
  var typeaheadRef;
  var state = {
    groupedItems: [],
    typeaheadPrefix: '',
    typeaheadContext: '',
    typeaheadText: ''
  };
  var handleTypeaheadDebounced = debounce(handleTypeahead, TYPEAHEAD_DEBOUNCE);

  var setState = function setState(update) {
    state = _objectSpread(_objectSpread({}, state), update);
  };

  return {
    onBlur: function onBlur(event, editor, next) {
      state = _objectSpread(_objectSpread({}, state), {}, {
        groupedItems: []
      });
      return next();
    },
    onClick: function onClick(event, editor, next) {
      state = _objectSpread(_objectSpread({}, state), {}, {
        groupedItems: []
      });
      return next();
    },
    onKeyDown: function onKeyDown(event, editor, next) {
      var keyEvent = event;
      var currentSuggestions = state.groupedItems;
      var hasSuggestions = currentSuggestions.length;

      switch (keyEvent.key) {
        case 'Escape':
          {
            if (hasSuggestions) {
              keyEvent.preventDefault();
              state = _objectSpread(_objectSpread({}, state), {}, {
                groupedItems: []
              }); // Bogus edit to re-render editor

              return editor.insertText('');
            }

            break;
          }

        case 'ArrowDown':
        case 'ArrowUp':
          if (hasSuggestions) {
            keyEvent.preventDefault();
            typeaheadRef.moveMenuIndex(keyEvent.key === 'ArrowDown' ? 1 : -1);
            return;
          }

          break;

        case 'Enter':
          {
            if (!(keyEvent.shiftKey || keyEvent.ctrlKey) && hasSuggestions) {
              keyEvent.preventDefault();
              return typeaheadRef.insertSuggestion();
            }

            break;
          }

        case 'Tab':
          {
            if (hasSuggestions) {
              keyEvent.preventDefault();
              return typeaheadRef.insertSuggestion();
            }

            break;
          }

        default:
          {
            // Don't react on meta keys
            if (keyEvent.key.length === 1) {
              handleTypeaheadDebounced(editor, setState, onTypeahead, cleanText);
            }

            break;
          }
      }

      return next();
    },
    commands: {
      selectSuggestion: function selectSuggestion(editor, suggestion) {
        var suggestions = state.groupedItems;

        if (!suggestions || !suggestions.length) {
          return editor;
        } // @ts-ignore


        var ed = editor.applyTypeahead(suggestion);
        handleTypeaheadDebounced(editor, setState, onTypeahead, cleanText);
        return ed;
      },
      applyTypeahead: function applyTypeahead(editor, suggestion) {
        var suggestionText = suggestion.insertText || suggestion.label;
        var preserveSuffix = suggestion.kind === 'function';
        var move = suggestion.move || 0;
        var _state = state,
            typeaheadPrefix = _state.typeaheadPrefix,
            typeaheadText = _state.typeaheadText,
            typeaheadContext = _state.typeaheadContext;

        if (onWillApplySuggestion) {
          suggestionText = onWillApplySuggestion(suggestionText, {
            groupedItems: state.groupedItems,
            typeaheadContext: typeaheadContext,
            typeaheadPrefix: typeaheadPrefix,
            typeaheadText: typeaheadText
          });
        } // Remove the current, incomplete text and replace it with the selected suggestion


        var backward = suggestion.deleteBackwards || typeaheadPrefix.length;
        var text = cleanText ? cleanText(typeaheadText) : typeaheadText;
        var suffixLength = text.length - typeaheadPrefix.length;
        var offset = typeaheadText.indexOf(typeaheadPrefix);
        var midWord = typeaheadPrefix && (suffixLength > 0 && offset > -1 || suggestionText === typeaheadText);
        var forward = midWord && !preserveSuffix ? suffixLength + offset : 0; // If new-lines, apply suggestion as block

        if (suggestionText.match(/\n/)) {
          var fragment = makeFragment(suggestionText);
          return editor.deleteBackward(backward).deleteForward(forward).insertFragment(fragment).focus();
        }

        state = _objectSpread(_objectSpread({}, state), {}, {
          groupedItems: []
        });
        return editor.deleteBackward(backward).deleteForward(forward).insertText(suggestionText).moveForward(move).focus();
      }
    },
    renderEditor: function renderEditor(props, editor, next) {
      if (editor.value.selection.isExpanded) {
        return next();
      }

      var children = next();
      return /*#__PURE__*/React.createElement(React.Fragment, null, children, /*#__PURE__*/React.createElement(Typeahead, {
        menuRef: function menuRef(menu) {
          return typeaheadRef = menu;
        },
        origin: portalOrigin,
        prefix: state.typeaheadPrefix,
        isOpen: !!state.groupedItems.length,
        groupedItems: state.groupedItems,
        onSelectSuggestion: editor.selectSuggestion
      }));
    }
  };
}

var handleTypeahead = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(editor, onStateChange, onTypeahead, cleanText) {
    var value, selection, parentBlock, selectionStartOffset, decorations, filteredDecorations, labelKeyDec, labelKey, wrapperClasses, text, prefix, labelValueMatch, _yield$onTypeahead, suggestions, context, filteredSuggestions;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (onTypeahead) {
              _context.next = 2;
              break;
            }

            return _context.abrupt("return");

          case 2:
            value = editor.value;
            selection = value.selection; // Get decorations associated with the current line

            parentBlock = value.document.getClosestBlock(value.focusBlock.key);
            selectionStartOffset = value.selection.start.offset - 1;
            decorations = parentBlock && parentBlock.getDecorations(editor);
            filteredDecorations = decorations ? decorations.filter(function (decoration) {
              return decoration.start.offset <= selectionStartOffset && decoration.end.offset > selectionStartOffset && decoration.type === TOKEN_MARK;
            }).toArray() : []; // Find the first label key to the left of the cursor

            labelKeyDec = decorations && decorations.filter(function (decoration) {
              return decoration.end.offset <= selectionStartOffset && decoration.type === TOKEN_MARK && decoration.data.get('className').includes('label-key');
            }).last();
            labelKey = labelKeyDec && value.focusText.text.slice(labelKeyDec.start.offset, labelKeyDec.end.offset);
            wrapperClasses = filteredDecorations.map(function (decoration) {
              return decoration.data.get('className');
            }).join(' ').split(' ').filter(function (className) {
              return className.length;
            });
            text = value.focusText.text;
            prefix = text.slice(0, selection.focus.offset);

            if (filteredDecorations.length) {
              text = value.focusText.text.slice(filteredDecorations[0].start.offset, filteredDecorations[0].end.offset);
              prefix = value.focusText.text.slice(filteredDecorations[0].start.offset, selection.focus.offset);
            } // Label values could have valid characters erased if `cleanText()` is
            // blindly applied, which would undesirably interfere with suggestions


            labelValueMatch = prefix.match(/(?:!?=~?"?|")(.*)/);

            if (labelValueMatch) {
              prefix = labelValueMatch[1];
            } else if (cleanText) {
              prefix = cleanText(prefix);
            }

            _context.next = 18;
            return onTypeahead({
              prefix: prefix,
              text: text,
              value: value,
              wrapperClasses: wrapperClasses,
              labelKey: labelKey || undefined,
              editor: editor
            });

          case 18:
            _yield$onTypeahead = _context.sent;
            suggestions = _yield$onTypeahead.suggestions;
            context = _yield$onTypeahead.context;
            filteredSuggestions = suggestions.map(function (group) {
              if (!group.items) {
                return group;
              }

              var newGroup = _objectSpread({}, group);

              if (prefix) {
                // Filter groups based on prefix
                if (!group.skipFilter) {
                  newGroup.items = newGroup.items.filter(function (c) {
                    return (c.filterText || c.label).length >= prefix.length;
                  });

                  if (group.prefixMatch) {
                    newGroup.items = newGroup.items.filter(function (c) {
                      return (c.filterText || c.label).startsWith(prefix);
                    });
                  } else {
                    newGroup.items = newGroup.items.filter(function (c) {
                      return (c.filterText || c.label).includes(prefix);
                    });
                  }
                } // Filter out the already typed value (prefix) unless it inserts custom text not matching the prefix


                newGroup.items = newGroup.items.filter(function (c) {
                  var _c$filterText;

                  return !(c.insertText === prefix || ((_c$filterText = c.filterText) !== null && _c$filterText !== void 0 ? _c$filterText : c.label) === prefix);
                });
              }

              if (!group.skipSort) {
                newGroup.items = sortBy(newGroup.items, function (item) {
                  return item.sortText || item.label;
                });
              }

              return newGroup;
            }).filter(function (gr) {
              return gr.items && gr.items.length;
            }); // Filter out empty groups

            onStateChange({
              groupedItems: filteredSuggestions,
              typeaheadPrefix: prefix,
              typeaheadContext: context,
              typeaheadText: text
            }); // Bogus edit to force re-render

            editor.blur().focus();

          case 24:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function handleTypeahead(_x, _x2, _x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();